# README

This folder contains four labs, written in Java, designed for an object oriented design class (e.g. CS2).

## Folder Contents

Package contents are as follows.

```bash
README.md #this README
lab1:
	lab1.pdf #spec for lab 1 (Basic OO design)
	starter_code #starter files for lab 1 (release these with the spec)
	example_submissions #example student submissions (good, mediocre and bad). The 'good' student files represent a lab solution. Each student submission contains an example summary of marks, generated by the automarker. Can be used to calibrate marking.
	supporting_files #supporting files for lab 1 (including test cases for marking and a template for results summary).
	README-lab1.md: #lab 1 specific README
	results.csv: #example marks (generated by automarker) for example students (good, mediocre and bad).
lab2:
	lab-setup-tutorial.pdf #setup tutorial for lab 2 (GUI design)
	lab2.pdf #spec for lab 2 (GUI design)	
	starter_code #starter files for lab 2 (release these with the spec)
	example_submissions #example student submissions (good, mediocre and bad). The 'good' student files represent a lab solution. Each student submission contains an example summary of marks, generated by the automarker. Can be used to calibrate marking.
	supporting_files #supporting files for lab 2 (including test cases for marking and a template for results summary).
	README-lab2.md: #lab 2 specific README
	results.csv: #example marks (generated by automarker) for example students (good, mediocre and bad).
	screenreader_files: #example solution containing javax.accessibility hooks to screenreader
lab2:
	lab3.pdf #spec for lab 3 (Observer Pattern)
	starter_code #starter files for lab 3 (release these with the spec)
	example_submissions #example student submissions (good, mediocre and bad). The 'good' student files represent a lab solution. Each student submission contains an example summary of marks, generated by the automarker. Can be used to calibrate marking.
	supporting_files #supporting files for lab 2 (including test cases for marking and a template for results summary).
	README-lab3.md: #lab 3 specific README
	results.csv: #example marks (generated by automarker) for example students (good, mediocre and bad).
lab2:
	lab4.pdf #spec for lab 4 (Decorator Pattern)
	starter_code #starter files for lab 4 (release these with the spec)
	example_submissions #example student submissions (good, mediocre and bad). The 'good' student files represent a lab solution. Each student submission contains an example summary of marks, generated by the automarker. Can be used to calibrate marking.
	supporting_files #supporting files for lab 4 (including test cases for marking and a template for results summary).
	README-lab4.md: #lab 4 specific README
	results.csv: #example marks (generated by automarker) for example students (good, mediocre and bad).
shared_files:
	run_tests.sh #script to run JUnit5 (and TestFX) Tests for a set of submissions in any of the lab folders.  This script is for a Mac (zsh) and contains references to paths that may need to be adjusted for other machines.		
	tally_marks.py  #Python 3+ script to tally marks and generate reports for a set of submissions in any of the lab folders. Produces reports in each student folder and results.csv files.
	junit-platform-console-standalone-1.9.0.jar #jar required to run JUnit5 tests from the command line
```

## Marking a Submission

Note that the shell scrips have been written for and tested on Mac OS. <br />

<b>To run code and JUnit tests for lab 1, 3 and 4 you will need: </b><p />

Java version: 18, vendor: Oracle Corporation <br />
JUnit5: version 5.9.2 <br />

<b>Tests for lab 2 (which is a GUI programming lab) include GUI tests (written with TestFX). These require: </b><p />

Apache Maven: version 3.6.3 <br />
Maven Surefire Plugin: version 2.22.0 <br />
JavaFX: version 20-ea+4 <br />
TestFX: version 4.0.16-alpha <br />

<b>To tally marks you will need: </b><p />

Python 3.9.6, and the following imports <br />
```python
import os
import re
import string
import sys
```
<p />

<b>To run the automarker you will need: </b><p />

zsh (on Mac by default)<br />

```bash
cd shared_files/ #to change into the directory with scripts to run tests 
chmod 775 run_tests.sh #to ensure marking script is runnable (on a Mac!)

./run_tests.sh lab-1 #to run tests on Lab 1
python3 tally_marks.py lab-1 #to tally Lab 1 marks and generate reports

./run_tests.sh lab-2 #to run tests on Lab 2
python3 tally_marks.py lab-2 #to tally Lab 2 marks and generate reports

./run_tests.sh lab-3 #to run tests on Lab 3
python3 tally_marks.py lab-3 #to tally Lab 3 marks and generate reports

./run_tests.sh lab-4 #to run tests on Lab 4
python3 tally_marks.py lab-4 #to tally Lab 4 marks and generate reports

```

## For More Information, Contact

Sonya Allin
University of Toronto Mississauga
s.allin@utoronto.ca

