/**
 * Class WhiteAsh.
 * A  White Ash is a tree that grows and stores carbon.
 *  - A WhiteAsh has a diameter
 *  - A WhiteAsh may be alive or dead
 *  - A WhiteAsh has a default growth rate of 0.3 inches in diameter a year
 */
public class WhiteAsh implements Comparable<WhiteAsh>{

    /**
     * default growth rate (in inches) of an ash tree
     */    
	private final double DEFAULT_RATE = 0.3;

    /**
     * default diameter (in inches) of an ash tree
     */    	
	private double diameter;

    /**
     * determines if tree is living or dead
     */   	
	private boolean alive;

	/**
	 * Create a White Ash with the given diameter. A new tree is, by default, alive.
	 *
	 * @param diameter	The diameter of this tree
	 */
	public WhiteAsh(double diameter) {
		this.diameter = diameter;
		this.alive = true;
	}

	/**
	 * Return string representation of this tree
	 *
	 * @return String	The string representation of this tree
	 */
	@Override
	public String toString() {
		String s = "Diameter: " + this.diameter +
				", Carbon content: " + this.carbonContent() + " kg";
		return s;
	}

	/**
	 * Get the diameter of this tree
	 *
	 * @return	The diameter of this tree
	 */
	public double getDiameter() {
		return this.diameter;
	}

	/**
	 * Set the diameter of this tree
	 */
	private void setDiameter(double diameter) {
		this.diameter = diameter;
	}

	/**
	 * Return True if this tree is alive and false otherwise.
	 *
	 * @return	Whether or not this tree is alive
	 */
	public boolean getLiving() {
		return this.alive;
	}

	/**
	 * Set the tree to be alive or dead
	 */
	private void setLiving(boolean b) { this.alive = b; }

	/**
	 * Increment the diameter of this tree based on the time (in years) that has elapsed
	 *  - If the tree is dead, do nothing
	 *  - If the time elapsed is less than 0, do nothing
	 *
	 * @param years  The number of years that have elapsed 
	 */
	public void grow(double years) {
		throw new UnsupportedOperationException("Method has yet to be implemented!");
	}


	/**
	 * Calculate the carbon content of this tree in kg, based on biomass
	 * Carbon content of a White Ash can be calculated using this formula:
	 * Carbon = a*B, where B is the tree's Biomass and a = 0.474.  We use
	 * a = 0.474 as this was the mean wood carbon fraction for tree species 
	 * surveyed by the USDA Forest Service Forest Inventory and Analysis (FIA) 
	 * program (https://www.fs.usda.gov/nrs/highlights/2256).
	 * 
	 * The tree's Biomass can be estimated using this formula:
	 * B = p*diameter^q, where diameter is the tree diameter.
	 * p = 0.19 and q = 2.17
	 * 
	 * The variables p and q are called allometric coefficeints.
	 * Allometric coefficeints for the White Ash and other tree species can be found here:
	 * https://www.researchgate.net/publication/249535320_Canadian_national_tree_aboveground_biomass_equations
	 *
	 * @return double  The kg of carbon sequestered in the wood of the tree
	 */
	public double carbonContent() {
		throw new UnsupportedOperationException("Method has yet to be implemented!");

	}


	/**
	 * Fell the tree. To fell (or chop down) the tree:
	 * - Sets the diameter to zero
	 * - Set the living status to 'false'.
	 */
	public void fellTree() {
		throw new UnsupportedOperationException("Method has yet to be implemented!");

	}


	/**
	 * Assess if two trees are equal. Two trees are equal if:
	 * - They have the same diameter
	 * - They are both alive or both dead
	 *
	 * @return boolean	True if the trees are equal
	 */
	@Override
	public boolean equals(Object other) {
		throw new UnsupportedOperationException("Method has yet to be implemented!");
	}

	/**
	 * Generate a unique hashCode for this tree.
	 *
	 * @return int	A hash code that is unique to all "equal" trees
	 */
	@Override
	public int hashCode() {
		throw new UnsupportedOperationException("Method has yet to be implemented!");
	}

	/**
	 * Compare this tree to another.
	 * - If one tree is alive and the other dead, the living tree is "greater" than the dead one
	 * - If both are alive or dead, the one with the larger diameter is "greater"
	 * - If the trees are equal, return 0.
	 *
	 * @return int	0,1 or -1 (meaning same, greater than or less than)
	 */
	@Override
	public int compareTo(WhiteAsh other) {
		throw new UnsupportedOperationException("Method has yet to be implemented!");
	}

}
