import javafx.event.EventHandler;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.TextField;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import treetools.MunicipalTree;

import java.util.Stack;
import java.util.List;

public class TreeFilterEventHandler implements EventHandler<MouseEvent> {

    private ChoiceBox treeSelect;
    private TextField txtSummary;
    private TreeViewer treeView;

    public TreeFilterEventHandler(ChoiceBox treeSelect, TextField txtSummary, TreeViewer view) {
        this.treeSelect = treeSelect;
        this.txtSummary = txtSummary;
        this.treeView = view;
    }

    @Override
    public void handle(MouseEvent event) {
        try {
            String treeType = (String) this.treeSelect.getValue();
            filterTrees(treeType);
        } catch (Exception e) {
            System.err.println("Error encountered:" + e);
        }
    }

    public void filterTrees(String treeType) {
        Stack undoStack = this.treeView.getUndoStack();
        List<MunicipalTree> trees = this.treeView.getTrees();
        Circle circle;
        while (!undoStack.empty()) { //clear all the circles
            circle = (Circle) undoStack.pop();
            treeView.getAnchorRoot().getChildren().remove(circle);
        }
        int h = treeView.getHeight();
        int w = treeView.getWidth();
        double[] boundaries = treeView.getBoundaries();

        for (MunicipalTree t : trees) {
            String name = t.getName();
            if (name.equals(treeType) || treeType.equals("ALL TREES")) {
                double[] coords = t.getLoc().getCoords();
                double yval = (double) h - h * ((coords[1] - boundaries[0]) / (boundaries[2] - boundaries[0]));
                double xval = (double) w - w * ((coords[0] - boundaries[1]) / (boundaries[3] - boundaries[1]));
                if (yval < h & yval > 0 & xval < w & xval > 0) {
                    circle = new Circle();
                    circle.setCenterX(xval);
                    circle.setCenterY(yval);
                    circle.setRadius(3);
                    circle.setFill(Color.RED);
                    treeView.getAnchorRoot().getChildren().add(circle);
                    undoStack.add(circle);
                }
            }
        }

        this.txtSummary.setText("Trees selected: " + undoStack.size());

    }
}
