import os
import re
import string
import sys

# You may want to configure this to add or remove tests as you see fit
# values are lists of [[Hidden Tests], [Student Tests], [GUI Tests (which are hidden)]] for each lab
lab_tests = {'lab-1': [['currCarbonContentHiddenTest[^\n]*✘','growHiddenTest[^\n]*✘','carbonContentHiddenTest[^\n]*✘',
                       'fellTreeHiddenTest[^\n]*✘','equalHiddenTest[^\n]*✘','notEqualHiddenTest[^\n]*✘','equalsDifferentObjHiddenTest[^\n]*✘',
                       'hashCodeEqualHiddenTest[^\n]*✘','hashCodeUnequalHiddenTest[^\n]*✘','compareToHiddenTest[^\n]*✘'],
                        ['carbonContentTreeTest[^\n]*✘', 'growTest[^\n]*✘', 'currCarbonContentForestTest[^\n]*✘'],
                        []],
              'lab-2': [['getTreeTypeHiddenTest[^\n]*✘','countTreeHiddenTest[^\n]*✘', 'countTreeHidden2Test[^\n]*✘', 'countTreeHidden3Test[^\n]*✘', 'countTreeHidden4Test[^\n]*✘'],
                        ['getTreeTypeTest[^\n]*✘', 'countTreeTest[^\n]*✘'],
                        ['\[ERROR\]   TreeViewerTest.test_NodeCount','\[ERROR\]   TreeViewerTest.test_StartProgram']],
              'lab-3': [['registerHiddenTest[^\n]*✘', 'unregisterHiddenTest[^\n]*✘','notifyObserversHiddenTest[^\n]*✘',
                           'addGuidelineHiddenTest[^\n]*✘','removeGuidelineHiddenTest[^\n]*✘', 'updateHiddenTest[^\n]*✘'],
                        ['addGuidelineTest[^\n]*✘', 'removeGuidelineTest[^\n]*✘', 'updateTest[^\n]*✘'],
                        []],
              'lab-4': [['TreeDecoratorFirstHiddenTest[^\n]*✘', 'TreeDecoratorSecondHiddenTest[^\n]*✘','MixedHiddenTest[^\n]*✘'],
                        ['FoliageDecoratorStudentTest[^\n]*✘','BarkDecoratorStudentTest[^\n]*✘'],
                        []]
                }

def summarize(folder):
    name = folder[-5:]
    directory = folder + '/example_submissions'
    items = []
    for item in os.listdir(directory):
        if os.path.isdir(os.path.join(directory, item)):
            items.append(item)

    template = open(folder + '/supporting_files/results_template.txt')
    template = template.read()

    itemdic = {}
    for i in items:

        print("Summarizing results for: " + i)
        compile_error = False

        formatted = template
        formatted = formatted.replace("{}", i, 1)

        hidden_tests = lab_tests[name][0]
        student_tests = lab_tests[name][1]
        gui_tests = lab_tests[name][2]
        h_total, s_total, g_total = len(hidden_tests), len(student_tests), len(gui_tests)
        formatted = formatted.replace("{}", str(h_total + s_total + g_total), 1)

        #i = i.translate(str.maketrans('', '', string.punctuation))
        cfile = directory + '/' + i + '/compile_errors.txt' #if there are compile errors
        rfile = directory + '/' + i + '/junit_test_results.txt' #if there are junit test results
        gfile = directory + '/' + i + '/gui_test_results.txt' #if there are gui test results
        guiresults, errors, results = None, None, None

        #are there GUI tests?
        if (g_total > 0):
            if (not os.path.exists(gfile)):
                print(gfile + " doesn't exist!!")  # no results here!
            else:
                gfile = open(gfile)
                guiresults = gfile.read()
                gfile.close()
                match = re.search("COMPILATION ERROR", guiresults)
                if not match: errors = [] #if there's success, is all good
                else: # compile errors for GUI tests
                    compile_error = True
                    itemdic[i] = (0, s_total, 0, 0, (h_total + g_total), 0)                

        #are there JUNIT tests? 
        if (not os.path.exists(rfile)):
            print(rfile + " doesn't exist!!") #no results here!
        else:
            cfile = open(cfile)
            errors = cfile.read()
            cfile.close()

            if len(errors) > 3: # are there compile errors other than in the GUI files?
                itemdic[i] = (0, s_total, 0, 0, h_total, 0)
                for j in range(0, len(itemdic[i])):
                    formatted = formatted.replace("{}", str(itemdic[i][j]), 1)
                formatted = formatted.replace("{}","Compilation errors resulted in a mark of 0. Errors are below:\n\n" + errors, 1)

            else:
                block_search1 = re.compile('Thanks for using.*0m',re.DOTALL)
                block_search2 = re.compile('Failures .*Test run finished',re.DOTALL)
                score1, score2 = 0, 0

                with open(rfile,'r',encoding='utf-8') as f:

                    text = f.read()
                    for t in student_tests:
                        match = re.search(t, text)
                        if match is None:
                            score1 += 1

                    for t in hidden_tests:
                        match = re.search(t, text)
                        if match is None:
                            score2 += 1

                    result1 = re.findall(block_search1,text)
                    result2 = re.findall(block_search2,text)

                    for t in gui_tests: #if there are GUI tests, assume they are "hidden"
                        match = re.search(t, guiresults)
                        if match is None:
                            score2 += 1
                        h_total += 1

                if not compile_error:
                    itemdic[i] = (score1, s_total, round((score1/s_total)*2.5,2), score2, h_total, round((score2/h_total)*7.5,2))

                results = ""
                if len(result1) > 0: results += "\n************  JUNIT TEST RESULTS ****************\n\n" + result1[0]
                if len(result2) > 0: results += "\n****************************\n\n" + result2[0]
                if (g_total > 0): #are there gui tests?                    
                    matches = re.findall('\[ERROR\] [^\n]+\n', guiresults)
                    if (len(matches) > 0): 
                        results += "\n\n************  GUI TEST ERRORS ****************\n\n"
                        for j in matches:
                            results += j
                    else:
                        results += "\n\n************  NO GUI TEST ERRORS ****************\n\n"

                for j in range(0, len(itemdic[i])):
                    formatted = formatted.replace("{}", str(itemdic[i][j]), 1)
                formatted = formatted.replace("{}", str(itemdic[i][2] + itemdic[i][5]), 1)

                formatted = formatted.replace("{}", "Details produced by the automarker are below:\n\n" + results, 1)

            outfile = open(directory + '/' + i + '/report.txt', 'w')
            outfile.write(formatted)
            outfile.close()

    # print out a results summary
    scores = open(folder + '/results.csv', 'w')
    scores.write("Student ID, Student Tests Passed, Student Tests Count, Student Total Score, Hidden Tests Passed, Hidden Tests Count, Hidden Total Score, Overall Total\n")
    for key, value in itemdic.items():
        result = ','.join([str(i) for i in value[0:6]])
        scores.write(str(key) + ", " + result + ", " + str(value[2]+value[5]) + "\n")
    scores.close()

if __name__ == "__main__":
   if(sys.argv[1][0] != '.'): sys.argv[1] = "../" + sys.argv[1]
   summarize(sys.argv[1])





